/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.zbdsTable.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.zbdsTable.entity.ZbdsTableEntity;
import org.springblade.modules.zbdsTable.vo.ZbdsTableVO;
import org.springblade.modules.zbdsTable.wrapper.ZbdsTableWrapper;
import org.springblade.modules.zbdsTable.service.IZbdsTableService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 装备丢失表 控制器
 *
 * @author BladeX
 * @since 2024-06-06
 */
@RestController
@AllArgsConstructor
@RequestMapping("blade-zbdsTable/zbdsTable")
@Api(value = "装备丢失表", tags = "装备丢失表接口")
public class ZbdsTableController extends BladeController {

	private final IZbdsTableService zbdsTableService;

	/**
	 * 装备丢失表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入zbdsTable")
	public R<ZbdsTableVO> detail(ZbdsTableEntity zbdsTable) {
		ZbdsTableEntity detail = zbdsTableService.getOne(Condition.getQueryWrapper(zbdsTable));
		return R.data(ZbdsTableWrapper.build().entityVO(detail));
	}
	/**
	 * 装备丢失表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入zbdsTable")
	public R<IPage<ZbdsTableVO>> list(ZbdsTableEntity zbdsTable, Query query) {
		IPage<ZbdsTableEntity> pages = zbdsTableService.page(Condition.getPage(query), Condition.getQueryWrapper(zbdsTable));
		return R.data(ZbdsTableWrapper.build().pageVO(pages));
	}

	/**
	 * 装备丢失表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入zbdsTable")
	public R<IPage<ZbdsTableVO>> page(ZbdsTableVO zbdsTable, Query query) {
		IPage<ZbdsTableVO> pages = zbdsTableService.selectZbdsTablePage(Condition.getPage(query), zbdsTable);
		return R.data(pages);
	}

	/**
	 * 装备丢失表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入zbdsTable")
	public R save(@Valid @RequestBody ZbdsTableEntity zbdsTable) {
		return R.status(zbdsTableService.save(zbdsTable));
	}

	/**
	 * 装备丢失表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入zbdsTable")
	public R update(@Valid @RequestBody ZbdsTableEntity zbdsTable) {
		return R.status(zbdsTableService.updateById(zbdsTable));
	}

	/**
	 * 装备丢失表 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入zbdsTable")
	public R submit(@Valid @RequestBody ZbdsTableEntity zbdsTable) {
		return R.status(zbdsTableService.saveOrUpdate(zbdsTable));
	}

	/**
	 * 装备丢失表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(zbdsTableService.deleteLogic(Func.toLongList(ids)));
	}


}
